/***********************************************************************/
/***********************************************************************/
/** aurora224v1 mixer project (www.auroramixer.com):                  **/
/**  aurora is matt aldrich, mike garbus, & maro scaicchitano         **/
/**                                                                   **/
/** aurora_u1_18f4525_rev1.c     (version 1)                          **/
/** author: m.garbus 2008                                             **/
/** description:                                                      **/
/**  - for pic 18lf4525 (component u1) using a 20mhz crystal          **/
/**  - written for sourceboost boostc compiler (www.sourceboost.com)  **/
/**  - config bits:  address  |  value                                **/
/**                   300001  |  0x02                                 **/
/**                   300002  |  0x18                                 **/
/**                   300003  |  0x10                                 **/
/**                   300005  |  0x03                                 **/
/**                   300006  |  0x81                                 **/
/**                   300008  |  0x07                                 **/
/**                   300009  |  0xC0                                 **/
/**                   30000A  |  0x07                                 **/
/**                   30000B  |  0xE0                                 **/
/**                   30000C  |  0x07                                 **/
/**                   30000D  |  0x40                                 **/
/**  - this code is designed to read the state of the rotary          **/
/**    controllers, sliders, and buttons. the information is then     **/
/**    sent out of the uart at 57.6k baud. the rotary and sliders are **/
/**    measured using the adc. the information is measured at 7-bits  **/
/**    of resolution in compliance with midi. the main loop           **/
/**    continually detects changes, even while servicing interrupts.  **/
/**    when changes are detected, each change is sent in an           **/
/**    individual packet of the form:                                 **/
/**                   {0x55 0x55 CC VV}                               **/
/**    where CC is the midi continuous controller (cc) value and VV   **/
/**    is the 7-bit value. when changes are detected, they are saved  **/
/**    in a series of arrays, which the isr then parses to send the   **/
/**    packet. note that the most current measured value of any       **/
/**    change will be sent, not the value when the change was         **/
/**    detected. this allows for slightly faster updating.            **/
/***********************************************************************/
/***********************************************************************/

#include <system.h>
#include "aurora_u1_18f4525_rev1_rotmem.h"
#include "aurora_u1_18f4525_rev1_slidemem.h"


/* Set Config Bits */
#pragma DATA  _CONFIG1H, 0x02
#pragma DATA  _CONFIG2L, 0x18
#pragma DATA  _CONFIG2H, 0x10
#pragma DATA  _CONFIG3H, 0x03
#pragma DATA  _CONFIG4L, 0x81
#pragma DATA  _CONFIG5L, 0x07
#pragma DATA  _CONFIG5H, 0xC0
#pragma DATA  _CONFIG6L, 0x07
#pragma DATA  _CONFIG6H, 0xE0
#pragma DATA  _CONFIG7L, 0x07
#pragma DATA  _CONFIG7H, 0x40



#define TRUE  1
#define FALSE 0



/* Map Rotary Knobs: ((Mux # - 1) flipped) per Table II of ADG732 Spec */
#define ROTARY1  224 //(7 ) flipped
#define ROTARY2  96  //(6 ) flipped
#define ROTARY3  112 //(14) flipped
#define ROTARY4  200 //(19) flipped
#define ROTARY5  40  //(20) flipped
#define ROTARY6  24  //(24) flipped
#define ROTARY7  32  //(4 ) flipped
#define ROTARY8  192 //(3 ) flipped
#define ROTARY9  160 //(5 ) flipped
#define ROTARY10 136 //(17) flipped
#define ROTARY11 72  //(18) flipped
#define ROTARY12 168 //(21) flipped
#define ROTARY13 208 //(11) flipped
#define ROTARY14 48  //(12) flipped
#define ROTARY15 176 //(13) flipped
#define ROTARY16 216 //(27) flipped
#define ROTARY17 88  //(26) flipped
#define ROTARY18 152 //(25) flipped
#define ROTARY19 16  //(8 ) flipped
#define ROTARY20 80  //(10) flipped
#define ROTARY21 144 //(9 ) flipped
#define ROTARY22 8   //(16) flipped
#define ROTARY23 104 //(22) flipped
#define ROTARY24 232 //(23) flipped
/* Map Sliders: ((Mux # - 1) << 3) per Table II of ADG732 Spec */
#define SLIDER1 128 //(1 ) flipped (CC25)
#define SLIDER2 0   //(0 ) flipped (CC26)
#define SLIDER3 64  //(2 ) flipped (CC27)


/* Define Constant Array of Rotary Values */
/* changing order changes the knob/cc mapping */
const unsigned char Rotary_EncodedCH [NumOfRotary] =
{       ROTARY1,
        ROTARY2,
        ROTARY3,
        ROTARY4,
        ROTARY5,
        ROTARY6,
        ROTARY7,
        ROTARY8,
        ROTARY9,
        ROTARY10,
        ROTARY11,
        ROTARY12,
        ROTARY13,
        ROTARY14,
        ROTARY15,
        ROTARY16,
        ROTARY17,
        ROTARY18,
        ROTARY19,
        ROTARY20,
        ROTARY21,
        ROTARY22,
        ROTARY23,
        ROTARY24};

const unsigned char Slider_EncodedCH [NumOfSlider] =
{       SLIDER1,
        SLIDER2,
        SLIDER3};


/* Function Prototypes */
/***********************/
void reg_init(void);
unsigned char read_ADG732_channel_8bit (unsigned char ADG732_encoded_channel_num);
unsigned char ReadRotary_DetectChange_rev8bit_ave (void);
unsigned char ReadSlider_DetectChange_rev8bit_ave (void);
unsigned char ReadButton_DetectChange_StateMachine (void);

/* Global Variables */
/********************/
unsigned int  uint_counter1;
unsigned char RotaryState[NumOfRotary][3];
unsigned char SliderState[NumOfSlider][3];
unsigned char CurrentlyInTx_flag;
unsigned char RotaryStateChangeFlagCopy[NumOfRotary];
unsigned char SliderStateChangeFlagCopy[NumOfSlider];
unsigned char NewDataToSend_flag;
unsigned char PacketByteCount_ToBeTx;
//comms variables
unsigned char comms_BytesTransmitted;
unsigned char comms_ChangeFlagCounter;
unsigned char comms_ElementMarker;
unsigned char comms_MarkerFlag;
unsigned char comms_CurrentlyInRotaryValueTx;
unsigned char comms_CurrentlyInSliderValueTx;
unsigned char comms_CurrentlyInButtonValueTx;
unsigned char comms_CurrentlyInRotaryValueTxCounter;
unsigned char comms_CurrentlyInSliderValueTxCounter;
unsigned char comms_CurrentlyInButtonValueTxCounter;
unsigned char comms_AlreadySendingAValue;
//buttons
unsigned char ButtonState[NumOfButton][4];
unsigned char ButtonStateChangeFlagCopy[NumOfButton];



/**********************************************************************/
/* Definition of Elements of RotaryState and SliderState:             */
/* [0] Current read ADC value                                         */
/* [1] Change detected flag (true or false)                           */
/* [2] Last ADC value sent in last packet                             */
/**********************************************************************/

/**********************************************************************/
/* Definition of Elements of ButtonState:                             */
/* [0] Current button state                                           */
/* [1] Change detected flag (true or false)                           */
/* [2] Last button state sent in last packet                          */
/* [3] Track falling/rising edge for press                            */
/**********************************************************************/



void main(void)
{
    unsigned char change_in_rotary = FALSE;
    unsigned char change_in_slider = FALSE;
    unsigned char change_in_button = FALSE;
    unsigned char change_detected = FALSE;
    unsigned char new_change_while_in_tx_flag = FALSE;
    unsigned char RotaryTxDataCount, SliderTxDataCount, ButtonTxDataCount;


    /***************************************************/
    /*  PIC Setup - Bootup Procedure                   */
    /***************************************************/

    rcsta.7 = FALSE;     // Disable UART
    reg_init();          // Initialize Registers
    rcsta.7 = TRUE;      // Enable UART
    txsta.5 = TRUE;      // Enable Tx


    //turn all button LEDs off
    portb.0 = 0;
    portc.0 = 0;
    portc.1 = 0;
    portc.2 = 0;
    portc.3 = 0;
    portc.4 = 0;
    portc.5 = 0;
    portb.1 = 0;

    //Enable ADG732
    portd.0 = 1;  //Disable
    portd.2 = 1;  //CS_inv
    portd.1 = 1;  //WR_inv
    portd.0 = 0;  //Enable

    //initialize rotary, slider, & button states
    for (uint_counter1 = 0; uint_counter1 < NumOfRotary; ++uint_counter1)
    {
        RotaryState[uint_counter1][0] = 0;
        RotaryState[uint_counter1][1] = 0;
        RotaryState[uint_counter1][2] = 0;
    }
    for (uint_counter1 = 0; uint_counter1 < NumOfSlider; ++uint_counter1)
    {
        SliderState[uint_counter1][0] = 0;
        SliderState[uint_counter1][1] = 0;
        SliderState[uint_counter1][2] = 0;
    }
    for (uint_counter1 = 0; uint_counter1 < NumOfButton; ++uint_counter1)
    {
        ButtonState[uint_counter1][0] = 0;
        ButtonState[uint_counter1][1] = 0;
        ButtonState[uint_counter1][2] = 0;
        ButtonState[uint_counter1][3] = 0;
    }

    //initialize flags
    CurrentlyInTx_flag = FALSE;
    change_detected    = FALSE;
    new_change_while_in_tx_flag = FALSE;
    NewDataToSend_flag = FALSE;


    //enable watchdog timer
    wdtcon.0 = TRUE;

    /***************************************************/
    /*  Main Loop                                      */
    /***************************************************/
    while (TRUE)
    {
        //if change occured while sending last string of packets
        //compare last ADC reading to what was transmitted
        if (CurrentlyInTx_flag == FALSE && new_change_while_in_tx_flag == TRUE)
        {

            //rotary comparison
            for (uint_counter1 = 0; uint_counter1 < NumOfRotary; ++uint_counter1)
            {
                if (RotaryState[uint_counter1][0] != RotaryState[uint_counter1][2])
                {
                    //new info found that needs to be tx
                    RotaryState[uint_counter1][1] = TRUE;
                    NewDataToSend_flag = TRUE;
                }
                else
                    RotaryState[uint_counter1][1] = FALSE;
            }
            //slider comparison
            for (uint_counter1 = 0; uint_counter1 < NumOfSlider; ++uint_counter1)
            {
                if (SliderState[uint_counter1][0] != SliderState[uint_counter1][2])
                {
                    //new info found that needs to be tx
                    SliderState[uint_counter1][1] = TRUE;
                    NewDataToSend_flag = TRUE;
                }
                else
                    SliderState[uint_counter1][1] = FALSE;
            }
            //button comparison
            for (uint_counter1 = 0; uint_counter1 < NumOfButton; ++uint_counter1)
            {
                if (ButtonState[uint_counter1][0] != ButtonState[uint_counter1][2])
                {
                    //new info found that needs to be tx
                    ButtonState[uint_counter1][1] = TRUE;
                    NewDataToSend_flag = TRUE;
                }
                else
                    ButtonState[uint_counter1][1] = FALSE;
            }
            new_change_while_in_tx_flag = FALSE;
        }

        //read channels, buttons & detect changes
        change_in_rotary = ReadRotary_DetectChange_rev8bit_ave();
        change_in_slider = ReadSlider_DetectChange_rev8bit_ave();
        change_in_button = ReadButton_DetectChange_StateMachine();
        change_detected = change_in_rotary | change_in_slider | change_in_button;

        if (change_detected == TRUE && CurrentlyInTx_flag == FALSE)
        {
            //trigger comms to tx changes
            NewDataToSend_flag = TRUE;

        }
        else if (change_detected == TRUE && CurrentlyInTx_flag == TRUE)
        {
            //note change for when sending current string of packets has completed
            new_change_while_in_tx_flag = TRUE;

        }


        //prep data to be sent in isr
        if (NewDataToSend_flag == TRUE)
        {
            //make copy of what values to send
            RotaryTxDataCount = 0;
            for (uint_counter1 = 0; uint_counter1 < NumOfRotary; ++uint_counter1)
            {
                RotaryStateChangeFlagCopy[uint_counter1] = RotaryState[uint_counter1][1];
                if (RotaryStateChangeFlagCopy[uint_counter1] == TRUE)
                {
                    ++RotaryTxDataCount;
                    RotaryState[uint_counter1][1] = FALSE;
                }
            }
            SliderTxDataCount = 0;
            for (uint_counter1 = 0; uint_counter1 < NumOfSlider; ++uint_counter1)
            {
                SliderStateChangeFlagCopy[uint_counter1] = SliderState[uint_counter1][1];
                if (SliderStateChangeFlagCopy[uint_counter1] == TRUE)
                {
                    ++SliderTxDataCount;
                    SliderState[uint_counter1][1] = FALSE;
                }
            }
            ButtonTxDataCount = 0;
            for (uint_counter1 = 0; uint_counter1 < NumOfButton; ++uint_counter1)
            {
                ButtonStateChangeFlagCopy[uint_counter1] = ButtonState[uint_counter1][1];
                if (ButtonStateChangeFlagCopy[uint_counter1] == TRUE)
                {
                    ++ButtonTxDataCount;
                    ButtonState[uint_counter1][1] = FALSE;
                }
            }

            //calculate number of bytes of packet
            PacketByteCount_ToBeTx = 4 * RotaryTxDataCount + 4 * SliderTxDataCount + 4 * ButtonTxDataCount;

            //enable the Tx system (enable interrupt)
            pie1.4 = TRUE;  //(TXIE)
        }

        //reset watchdog timer
        asm
        {
            CLRWDT;
        }

    }
}


void reg_init(void)
{
   /* ADC setup                              */
   /*----------------------------------------*/
   //   A/D INFO
   //   Tad min = .7us see pg 360
   //   Tacq min = 1.4us see pg 360
   //   Fosc = 40Mhz
   //   Fosc/32 = 1.25Mhz
   //   Tad = 32/Fosc = .8us
   //   Fosc/64 = 625Khz
   //   Tad = 64/Fosc = 1.6us
   /*----------------------------------------*/
   adcon0 = 0x00;
   adcon1 = 0x3E;
   adcon2 = 0xBE;

   /* Setup Input/Output Direction Registers */
   /*----------------------------------------*/
   trisa  = 0xFF;
   trisb  = 0xFC;
   trisc  = 0x00;
   trisd  = 0x00;
   trise  = 0x07;

   /* Setup Communication Related Registers */
   /*---------------------------------------*/
   spbrgh  = 0x00;  //20MHz 57.6kbaud
   spbrg   = 0x56;  //20MHz 57.6kbaud
   baudcon = 0x08;
   txsta   = 0x04;
   rcsta   = 0x80;  //Rx disabled

   /* Setup Timer Registers */
   /*-----------------------*/
   tmr0h = 0x00;
   tmr0l = 0x00;
   tmr1h = 0x00;
   tmr1l = 0x00;
   tmr2  = 0x00;
   tmr3h = 0x00;
   tmr3l = 0x00;
   t0con = 0x00;
   t1con = 0xC4;
   t2con = 0x7B;
   t3con = 0x00;
   pr2     = 0xFF;

   /* Setup Interrupt Related Registers */
   /*-----------------------------------*/
   intcon  = 0xC0; //High & Low Priorities
   intcon2 = 0x00;
   intcon3 = 0x00;
   ipr1    = 0x00;
   ipr2    = 0x00;
   pie1    = 0x00;
   pie2    = 0x00;
   pir1    = 0x00;
   pir2    = 0x00;
   rcon    = 0x00;

   /* Setup SPI Related Registers */
   /*-----------------------------*/
   sspcon1 = 0x10;
   sspstat = 0x40;

   /* Other Registers */
   /*******************/
   wdtcon = 0x00;


   return;

}




unsigned char read_ADG732_channel_8bit (unsigned char ADG732_encoded_channel_num)
{
    unsigned int temp_reading;

    //setup ADG732
    portd.2 = 0;
    portd.1 = 0;
    portd   = ADG732_encoded_channel_num;
    portd.1 = 1;


    //setup for PIC channel
    adcon0   = 0x00;
    adcon0.0 = TRUE; //(ADON)

    //clear adc reading
    adresh = 0x00;
    adresl = 0x00;

    //start reading
    adcon0.1 = TRUE;

    //wait for reading to complete
    while (adcon0.1 == TRUE)
    {}

    portd.2 = 1;

    return ((adresh << 6) | (adresl >> 2));  //shift to 8-bits (drop 2 lsb)
}



unsigned char ReadRotary_DetectChange_rev8bit_ave (void)
{
    /****************************************************************/
    /* This function obtains values of all rotary positions.  It    */
    /* also compares the newly obtained value with the previous     */
    /* value.  If a change is detected, this function will return   */
    /* a value of one; otherwise zero.  This function will also     */
    /* mark the channel that changed by setting the second element  */
    /* of the 2-D RotaryState array to one.  This function also     */
    /* reverses the number such that 255 -> 0 and 0 -> 255, etc.    */
    /****************************************************************/

    unsigned char MasterChange = 0;
    unsigned char temp_rotary_reading;
    unsigned char ArrayIndex, i;
    unsigned long sum = 0;

    for (ArrayIndex = 0; ArrayIndex < NumOfRotary; ++ArrayIndex)
    {
        //read ADC
        temp_rotary_reading = read_ADG732_channel_8bit(Rotary_EncodedCH[ArrayIndex]);
        #if ADC_Resolution == 7
         temp_rotary_reading = temp_rotary_reading >> 1;
        #endif
        //reverse value
        #if ADC_Resolution == 7
         temp_rotary_reading = (127 - temp_rotary_reading);
        #else
         temp_rotary_reading = (255 - temp_rotary_reading);
        #endif
        //record current reading into averager memory & accumulate sum
        sum = 0;
        for (i = AveDepth - 1; i > 0; --i)
        {
            RotaryMemory[ArrayIndex][i] = RotaryMemory[ArrayIndex][i-1];
            sum += (unsigned long) RotaryMemory[ArrayIndex][i];
        }
        RotaryMemory[ArrayIndex][0] = temp_rotary_reading;
        sum += (unsigned long) temp_rotary_reading;
        //calc average
        #if AveDepth == 1
        temp_rotary_reading = (unsigned char) sum;
        #elif AveDepth == 2
        temp_rotary_reading = (unsigned char) (sum >> 1);
        #elif AveDepth == 4
        temp_rotary_reading = (unsigned char) (sum >> 2);
        #elif AveDepth == 8
        temp_rotary_reading = (unsigned char) (sum >> 3);
        #elif AveDepth == 16
        temp_rotary_reading = (unsigned char) (sum >> 4);
        #elif AveDepth == 32
        temp_rotary_reading = (unsigned char) (sum >> 5);
        #else
        temp_rotary_reading = (unsigned char) (sum / AveDepth);
        #endif
        //store information & detect change
        if (temp_rotary_reading != RotaryState[ArrayIndex][0])
        {
            RotaryState[ArrayIndex][0] = temp_rotary_reading;    //record new reading
            RotaryState[ArrayIndex][1] = TRUE;                   //set CH change flag
            MasterChange = 1;                                    //set master change flag
        }
    }

    return MasterChange;
}


unsigned char ReadSlider_DetectChange_rev8bit_ave (void)
{
    /****************************************************************/
    /* This function obtains values of all slider positions.  It    */
    /* also compares the newly obtained value with the previous     */
    /* value.  If a change is detected, this function will return   */
    /* a value of one; otherwise zero.  This function will also     */
    /* mark the channel that changed by setting the second element  */
    /* of the 2-D RotaryState array to one. This function also      */
    /* reverses the number such that 255 -> 0 and 0 -> 255, etc.    */
    /****************************************************************/

    unsigned char MasterChange = 0;
    unsigned char temp_slider_reading;
    unsigned char ArrayIndex, i;
    unsigned long sum = 0;


    for (ArrayIndex = 0; ArrayIndex < NumOfSlider; ++ArrayIndex)
    {
        //read ADC
        temp_slider_reading = read_ADG732_channel_8bit(Slider_EncodedCH[ArrayIndex]);
        #if ADC_Resolution == 7
         temp_slider_reading = temp_slider_reading >> 1;
        #endif
        //reverse value
        #if ADC_Resolution == 7
         temp_slider_reading = (127 - temp_slider_reading);
        #else
         temp_slider_reading = (255 - temp_slider_reading);
        #endif
        //record current reading into averager memory & accumulate sum
        sum = 0;
        for (i = AveDepth - 1; i > 0; --i)
        {
            SliderMemory[ArrayIndex][i] = SliderMemory[ArrayIndex][i-1];
            sum += (unsigned long) SliderMemory[ArrayIndex][i];
        }
        SliderMemory[ArrayIndex][0] = temp_slider_reading;
        sum += (unsigned long) temp_slider_reading;
        //calc average
        #if AveDepth == 1
         temp_slider_reading = (unsigned char) sum;
        #elif AveDepth == 2
         temp_slider_reading = (unsigned char) (sum >> 1);
        #elif AveDepth == 4
         temp_slider_reading = (unsigned char) (sum >> 2);
        #elif AveDepth == 8
         temp_slider_reading = (unsigned char) (sum >> 3);
        #elif AveDepth == 16
         temp_slider_reading = (unsigned char) (sum >> 4);
        #elif AveDepth == 32
         temp_slider_reading = (unsigned char) (sum >> 5);
        #else
         temp_slider_reading = (unsigned char) (sum / AveDepth);
        #endif
        //store information & detect change
        if (temp_slider_reading != SliderState[ArrayIndex][0])
        {
            SliderState[ArrayIndex][0] = temp_slider_reading;   //record new reading
            SliderState[ArrayIndex][1] = TRUE;                  //set CH change flag
            MasterChange = 1;                                   //set master change flag
        }
    }

    return MasterChange;
}


unsigned char ReadButton_DetectChange_StateMachine (void)
{
    /****************************************************************/
    /* This function determines the state of the buttons and their  */
    /* indicator LEDs. As a debouncing technique, a button must be  */
    /* zero for [ButtonDebounceAveDepth] previous readings. Falling */
    /* edges are tracked and used to alternate the state of the LED */
    /****************************************************************/

    unsigned char MasterChange = 0;
    unsigned char CurrentButton = 0;
    unsigned char sum = 0;
    unsigned char i;

    //Button 1: RB2 RB1 (old RC7 proto)
    //---------------------------------
    CurrentButton = 1;
    --CurrentButton;
    sum = 0;
    for (i = ButtonDebounceAveDepth - 1; i > 0; --i)
    {
        ButtonMemory[CurrentButton][i] = ButtonMemory[CurrentButton][i-1];
        sum += ButtonMemory[CurrentButton][i];
    }
    ButtonMemory[CurrentButton][0] = portb.2;
    sum += ButtonMemory[CurrentButton][0];
    if (sum == 0)
    {
        if (ButtonState[CurrentButton][3] == FALSE)
        {
            //button pressed (averaged rising edge)
            ButtonState[CurrentButton][1] = TRUE;
            if (ButtonState[CurrentButton][0] == FALSE)
            {
                ButtonState[CurrentButton][0] = TRUE;
                portb.1 = 1;
            }
            else
            {
                ButtonState[CurrentButton][0] = FALSE;
                portb.1 = 0;
            }
            MasterChange = 1;
        }
        ButtonState[CurrentButton][3] = TRUE;
    }
    else
    {
        ButtonState[CurrentButton][3] = FALSE;
    }

    //Button 2: RB3 RB0
    //-----------------
    CurrentButton = 2;
    --CurrentButton;
    sum = 0;
    for (i = ButtonDebounceAveDepth - 1; i > 0; --i)
    {
        ButtonMemory[CurrentButton][i] = ButtonMemory[CurrentButton][i-1];
        sum += ButtonMemory[CurrentButton][i];
    }
    ButtonMemory[CurrentButton][0] = portb.3;
    sum += ButtonMemory[CurrentButton][0];
    if (sum == 0)
    {
        if (ButtonState[CurrentButton][3] == FALSE)
        {
            //button pressed (averaged rising edge)
            ButtonState[CurrentButton][1] = TRUE;
            if (ButtonState[CurrentButton][0] == FALSE)
            {
                ButtonState[CurrentButton][0] = TRUE;
                portb.0 = 1;
            }
            else
            {
                ButtonState[CurrentButton][0] = FALSE;
                portb.0 = 0;
            }
            MasterChange = 1;
        }
        ButtonState[CurrentButton][3] = TRUE;
    }
    else
    {
        ButtonState[CurrentButton][3] = FALSE;
    }

    //Button 3: RA1 RC0
    //-----------------
    CurrentButton = 3;
    --CurrentButton;
    sum = 0;
    for (i = ButtonDebounceAveDepth - 1; i > 0; --i)
    {
        ButtonMemory[CurrentButton][i] = ButtonMemory[CurrentButton][i-1];
        sum += ButtonMemory[CurrentButton][i];
    }
    ButtonMemory[CurrentButton][0] = porta.1;
    sum += ButtonMemory[CurrentButton][0];
    if (sum == 0)
    {
        if (ButtonState[CurrentButton][3] == FALSE)
        {
            //button pressed (averaged rising edge)
            ButtonState[CurrentButton][1] = TRUE;
            if (ButtonState[CurrentButton][0] == FALSE)
            {
                ButtonState[CurrentButton][0] = TRUE;
                portc.0 = 1;
            }
            else
            {
                ButtonState[CurrentButton][0] = FALSE;
                portc.0 = 0;
            }
            MasterChange = 1;
        }
        ButtonState[CurrentButton][3] = TRUE;
    }
    else
    {
        ButtonState[CurrentButton][3] = FALSE;
    }

    //Button 4: RA4 RC1
    //-----------------
    CurrentButton = 4;
    --CurrentButton;
    sum = 0;
    for (i = ButtonDebounceAveDepth - 1; i > 0; --i)
    {
        ButtonMemory[CurrentButton][i] = ButtonMemory[CurrentButton][i-1];
        sum += ButtonMemory[CurrentButton][i];
    }
    ButtonMemory[CurrentButton][0] = porta.4;
    sum += ButtonMemory[CurrentButton][0];
    if (sum == 0)
    {
        if (ButtonState[CurrentButton][3] == FALSE)
        {
            //button pressed (averaged rising edge)
            ButtonState[CurrentButton][1] = TRUE;
            if (ButtonState[CurrentButton][0] == FALSE)
            {
                ButtonState[CurrentButton][0] = TRUE;
                portc.1 = 1;
            }
            else
            {
                ButtonState[CurrentButton][0] = FALSE;
                portc.1 = 0;
            }
            MasterChange = 1;
        }
        ButtonState[CurrentButton][3] = TRUE;
    }
    else
    {
        ButtonState[CurrentButton][3] = FALSE;
    }

    //Button 5: RE0 RC3
    //-----------------
    CurrentButton = 5;
    --CurrentButton;
    sum = 0;
    for (i = ButtonDebounceAveDepth - 1; i > 0; --i)
    {
        ButtonMemory[CurrentButton][i] = ButtonMemory[CurrentButton][i-1];
        sum += ButtonMemory[CurrentButton][i];
    }
    ButtonMemory[CurrentButton][0] = porte.0;
    sum += ButtonMemory[CurrentButton][0];
    if (sum == 0)
    {
        if (ButtonState[CurrentButton][3] == FALSE)
        {
            //button pressed (averaged rising edge)
            ButtonState[CurrentButton][1] = TRUE;
            if (ButtonState[CurrentButton][0] == FALSE)
            {
                ButtonState[CurrentButton][0] = TRUE;
                portc.3 = 1;
            }
            else
            {
                ButtonState[CurrentButton][0] = FALSE;
                portc.3 = 0;
            }
            MasterChange = 1;
        }
        ButtonState[CurrentButton][3] = TRUE;
    }
    else
    {
        ButtonState[CurrentButton][3] = FALSE;
    }

    //Button 6: RA5 RC2
    //-----------------
    CurrentButton = 6;
    --CurrentButton;
    sum = 0;
    for (i = ButtonDebounceAveDepth - 1; i > 0; --i)
    {
        ButtonMemory[CurrentButton][i] = ButtonMemory[CurrentButton][i-1];
        sum += ButtonMemory[CurrentButton][i];
    }
    ButtonMemory[CurrentButton][0] = porta.5;
    sum += ButtonMemory[CurrentButton][0];
    if (sum == 0)
    {
        if (ButtonState[CurrentButton][3] == FALSE)
        {
            //button pressed (averaged rising edge)
            ButtonState[CurrentButton][1] = TRUE;
            if (ButtonState[CurrentButton][0] == FALSE)
            {
                ButtonState[CurrentButton][0] = TRUE;
                portc.2 = 1;
            }
            else
            {
                ButtonState[CurrentButton][0] = FALSE;
                portc.2 = 0;
            }
            MasterChange = 1;
        }
        ButtonState[CurrentButton][3] = TRUE;
    }
    else
    {
        ButtonState[CurrentButton][3] = FALSE;
    }

    //Button 7: RE2 RC5
    //-----------------
    CurrentButton = 7;
    --CurrentButton;
    sum = 0;
    for (i = ButtonDebounceAveDepth - 1; i > 0; --i)
    {
        ButtonMemory[CurrentButton][i] = ButtonMemory[CurrentButton][i-1];
        sum += ButtonMemory[CurrentButton][i];
    }
    ButtonMemory[CurrentButton][0] = porte.2;
    sum += ButtonMemory[CurrentButton][0];
    if (sum == 0)
    {
        if (ButtonState[CurrentButton][3] == FALSE)
        {
            //button pressed (averaged rising edge)
            ButtonState[CurrentButton][1] = TRUE;
            if (ButtonState[CurrentButton][0] == FALSE)
            {
                ButtonState[CurrentButton][0] = TRUE;
                portc.5 = 1;
            }
            else
            {
                ButtonState[CurrentButton][0] = FALSE;
                portc.5 = 0;
            }
            MasterChange = 1;
        }
        ButtonState[CurrentButton][3] = TRUE;
    }
    else
    {
        ButtonState[CurrentButton][3] = FALSE;
    }

    //Button 8: RE1 RC4
    //-----------------
    CurrentButton = 8;
    --CurrentButton;
    sum = 0;
    for (i = ButtonDebounceAveDepth - 1; i > 0; --i)
    {
        ButtonMemory[CurrentButton][i] = ButtonMemory[CurrentButton][i-1];
        sum += ButtonMemory[CurrentButton][i];
    }
    ButtonMemory[CurrentButton][0] = porte.1;
    sum += ButtonMemory[CurrentButton][0];
    if (sum == 0)
    {
        if (ButtonState[CurrentButton][3] == FALSE)
        {
            //button pressed (averaged rising edge)
            ButtonState[CurrentButton][1] = TRUE;
            if (ButtonState[CurrentButton][0] == FALSE)
            {
                ButtonState[CurrentButton][0] = TRUE;
                portc.4 = 1;
            }
            else
            {
                ButtonState[CurrentButton][0] = FALSE;
                portc.4 = 0;
            }
            MasterChange = 1;
        }
        ButtonState[CurrentButton][3] = TRUE;
    }
    else
    {
        ButtonState[CurrentButton][3] = FALSE;
    }

    return MasterChange;
}






void interrupt (void)
{
   //Tx Interrupt System - used to transmit a series of packets
   //isr transmits one byte at a time
   //most code is here to determine which byte should be transmitted during this interrupt
   if (pie1.4 && pir1.4) //(TXIE && TXIF)
   {
       CurrentlyInTx_flag = TRUE;

       //if transmitting new series of packets, initialize variables
       if (NewDataToSend_flag == TRUE)
       {
            comms_BytesTransmitted = 0;
            NewDataToSend_flag = FALSE;
            comms_CurrentlyInRotaryValueTx = FALSE;
            comms_CurrentlyInSliderValueTx = FALSE;
            comms_CurrentlyInButtonValueTx = FALSE;
            comms_CurrentlyInRotaryValueTxCounter = 0;
            comms_CurrentlyInSliderValueTxCounter = 0;
            comms_CurrentlyInButtonValueTxCounter = 0;
            comms_AlreadySendingAValue = FALSE;
       }


       if (comms_BytesTransmitted < PacketByteCount_ToBeTx)
       {
            //DEAL WITH ROTARY VALUES
            //-----------------------
            if (comms_CurrentlyInRotaryValueTx == FALSE && comms_CurrentlyInSliderValueTx == FALSE
                 && comms_CurrentlyInButtonValueTx == FALSE && comms_AlreadySendingAValue == FALSE)
            {

                 comms_CurrentlyInRotaryValueTxCounter = 0;

                  //find next rotary data value to tx, if any, according to RotaryStateChangeFlagCopy
                  comms_ChangeFlagCounter = 0;
                  comms_ElementMarker = 0;
                  comms_MarkerFlag = FALSE;
                  while (comms_ChangeFlagCounter < NumOfRotary && comms_MarkerFlag == FALSE)
                  {
                       if (RotaryStateChangeFlagCopy[comms_ChangeFlagCounter] != FALSE)
                       {
                           comms_MarkerFlag = TRUE;
                           comms_ElementMarker = comms_ChangeFlagCounter;
                           RotaryStateChangeFlagCopy[comms_ChangeFlagCounter] = FALSE;
                       }
                       ++comms_ChangeFlagCounter;
                  }
                  //if something was found, sent it; else, no more rotary values to send
                  if (comms_ChangeFlagCounter <= NumOfRotary && comms_MarkerFlag == TRUE)
                  {
                       comms_CurrentlyInRotaryValueTx = TRUE;

                       //send first sync 0x55
                       txreg = 0x55;

                       ++comms_CurrentlyInRotaryValueTxCounter;
                  }
                  else
                       comms_CurrentlyInRotaryValueTx = FALSE;

            }
            else if (comms_CurrentlyInRotaryValueTx == TRUE && comms_CurrentlyInRotaryValueTxCounter == 1)
            {
                   //send second sync 0x55
                   txreg = 0x55;

                   //log value that was sent
                   RotaryState[comms_ElementMarker][2] = RotaryState[comms_ElementMarker][0];

                   comms_CurrentlyInRotaryValueTx = TRUE;
                   comms_AlreadySendingAValue     = TRUE;

                   ++comms_CurrentlyInRotaryValueTxCounter;
            }
            else if (comms_CurrentlyInRotaryValueTx == TRUE && comms_CurrentlyInRotaryValueTxCounter == 2)
            {
                   //transmit current rotary CC #
                   txreg = comms_ElementMarker + 1;

                   comms_CurrentlyInRotaryValueTx = TRUE;
                   comms_AlreadySendingAValue     = TRUE;

                   ++comms_CurrentlyInRotaryValueTxCounter;
            }
            else if (comms_CurrentlyInRotaryValueTx == TRUE && comms_CurrentlyInRotaryValueTxCounter == 3)
            {
                   //aquire most current reading for this rotary & send
                   txreg = (RotaryState[comms_ElementMarker][0]);

                   //log value that was sent
                   RotaryState[comms_ElementMarker][2] = RotaryState[comms_ElementMarker][0];

                   comms_CurrentlyInRotaryValueTx = FALSE;
                   comms_AlreadySendingAValue     = TRUE;

                   ++comms_CurrentlyInRotaryValueTxCounter;
            }
            else if (comms_CurrentlyInRotaryValueTx == TRUE && comms_CurrentlyInRotaryValueTxCounter >= 4)
            {
                   //this case should not happen since there rotary values are only 8-bits
                   comms_CurrentlyInRotaryValueTx = FALSE;

                   ++comms_CurrentlyInRotaryValueTxCounter;
            }

            //DEAL WITH SLIDER VALUES
            //-----------------------
            if (comms_CurrentlyInRotaryValueTx == FALSE && comms_CurrentlyInSliderValueTx == FALSE
                && comms_CurrentlyInButtonValueTx == FALSE && comms_AlreadySendingAValue == FALSE)
            {

                   comms_CurrentlyInSliderValueTxCounter = 0;

                   //find next slider data value to tx, if any, according to SliderStateChangeFlagCopy
                   comms_ChangeFlagCounter = 0;
                   comms_ElementMarker = 0;
                   comms_MarkerFlag = FALSE;
                   while (comms_ChangeFlagCounter < NumOfSlider && comms_MarkerFlag == FALSE)
                   {
                       if (SliderStateChangeFlagCopy[comms_ChangeFlagCounter] != FALSE)
                       {
                           comms_MarkerFlag = TRUE;
                           comms_ElementMarker = comms_ChangeFlagCounter;
                           SliderStateChangeFlagCopy[comms_ChangeFlagCounter] = FALSE;
                       }
                       ++comms_ChangeFlagCounter;
                   }
                   //if something was found, sent it; else, no more slider values to send
                   if (comms_ChangeFlagCounter <= NumOfSlider && comms_MarkerFlag == TRUE)
                   {
                       comms_CurrentlyInSliderValueTx = TRUE;

                       //send first sync 0x55
                       txreg = 0x55;

                       ++comms_CurrentlyInSliderValueTxCounter;
                   }
                   else
                       comms_CurrentlyInSliderValueTx = FALSE;
            }
            else if (comms_CurrentlyInSliderValueTx == TRUE && comms_CurrentlyInSliderValueTxCounter == 1)
            {
                   //send second sync 0x55
                   txreg = 0x55;

                   comms_CurrentlyInSliderValueTx = TRUE;
                   comms_AlreadySendingAValue     = TRUE;

                   ++comms_CurrentlyInSliderValueTxCounter;
            }
            else if (comms_CurrentlyInSliderValueTx == TRUE && comms_CurrentlyInSliderValueTxCounter == 2)
            {

                   //transmit current slider CC #
                   txreg = comms_ElementMarker + 1 + 24;

                   comms_CurrentlyInSliderValueTx = TRUE;
                   comms_AlreadySendingAValue     = TRUE;

                   ++comms_CurrentlyInSliderValueTxCounter;
            }
            else if (comms_CurrentlyInSliderValueTx == TRUE && comms_CurrentlyInSliderValueTxCounter == 3)
            {

                   //aquire most current reading for this slider & send
                   txreg = (SliderState[comms_ElementMarker][0]);

                   //log value that was sent
                   SliderState[comms_ElementMarker][2] = SliderState[comms_ElementMarker][0];

                   comms_CurrentlyInSliderValueTx = FALSE;
                   comms_AlreadySendingAValue     = TRUE;

                   ++comms_CurrentlyInSliderValueTxCounter;
            }
            else if (comms_CurrentlyInSliderValueTx == TRUE && comms_CurrentlyInSliderValueTxCounter >= 4)
            {
                   //this case should not happen since slider values are only 8-bits
                   comms_CurrentlyInSliderValueTx = FALSE;

                   ++comms_CurrentlyInSliderValueTxCounter;
            }


            //DEAL WITH BUTTON VALUES
            //-----------------------
            if (comms_CurrentlyInRotaryValueTx == FALSE && comms_CurrentlyInSliderValueTx == FALSE
                && comms_CurrentlyInButtonValueTx == FALSE && comms_AlreadySendingAValue == FALSE)
            {

                   comms_CurrentlyInButtonValueTxCounter = 0;

                   //find next button data value to tx, if any, according to ButtonStateChangeFlagCopy
                   comms_ChangeFlagCounter = 0;
                   comms_ElementMarker = 0;
                   comms_MarkerFlag = FALSE;
                   while (comms_ChangeFlagCounter < NumOfButton && comms_MarkerFlag == FALSE)
                   {
                       if (ButtonStateChangeFlagCopy[comms_ChangeFlagCounter] != FALSE)
                       {
                           comms_MarkerFlag = TRUE;
                           comms_ElementMarker = comms_ChangeFlagCounter;
                           ButtonStateChangeFlagCopy[comms_ChangeFlagCounter] = FALSE;
                       }
                       ++comms_ChangeFlagCounter;
                   }
                   //if something was found, sent it; else, no more button values to send
                   if (comms_ChangeFlagCounter <= NumOfButton && comms_MarkerFlag == TRUE)
                   {
                       comms_CurrentlyInButtonValueTx = TRUE;

                       //send first sync 0x55
                       txreg = 0x55;

                       ++comms_CurrentlyInButtonValueTxCounter;
                   }
                   else
                       comms_CurrentlyInButtonValueTx = FALSE;
            }
            else if (comms_CurrentlyInButtonValueTx == TRUE && comms_CurrentlyInButtonValueTxCounter == 1)
            {
                   //send second sync 0x55
                   txreg = 0x55;

                   comms_CurrentlyInButtonValueTx = TRUE;
                   comms_AlreadySendingAValue     = TRUE;

                   ++comms_CurrentlyInButtonValueTxCounter;
            }
            else if (comms_CurrentlyInButtonValueTx == TRUE && comms_CurrentlyInButtonValueTxCounter == 2)
            {

                   //transmit current button CC #
                   txreg = comms_ElementMarker + 115;

                   comms_CurrentlyInButtonValueTx = TRUE;
                   comms_AlreadySendingAValue     = TRUE;

                   ++comms_CurrentlyInButtonValueTxCounter;
            }
            else if (comms_CurrentlyInButtonValueTx == TRUE && comms_CurrentlyInButtonValueTxCounter == 3)
            {

                   //aquire most current reading for this button & send scaled to 255
                   if (ButtonState[comms_ElementMarker][0] == 1)
                       #if ADC_Resolution == 8
                       txreg = 255;
                       #elif ADC_Resolution == 7
                       txreg = 127;
                       #else
                       txreg = 1;
                       #endif
                   else
                       txreg = 0;

                   //log value that was sent
                   ButtonState[comms_ElementMarker][2] = ButtonState[comms_ElementMarker][0];

                   comms_CurrentlyInButtonValueTx = FALSE;
                   comms_AlreadySendingAValue     = TRUE;

                   ++comms_CurrentlyInButtonValueTxCounter;
            }
            else if (comms_CurrentlyInButtonValueTx == TRUE && comms_CurrentlyInButtonValueTxCounter >= 4)
            {
                   //this case should not happen
                   comms_CurrentlyInButtonValueTx = FALSE;

                   ++comms_CurrentlyInButtonValueTxCounter;
            }
       }
       else
       {
            pie1.4 = 0; //(TXIE)
            CurrentlyInTx_flag = FALSE;
            comms_BytesTransmitted = 0;
       }


       ++comms_BytesTransmitted;
       comms_AlreadySendingAValue = FALSE;

       if (comms_BytesTransmitted == PacketByteCount_ToBeTx)
       {
            pie1.4 = 0; //(TXIE)
            CurrentlyInTx_flag = FALSE;
       }
   }

   return;
}


